/* Project: Robotik-Praktikum - "Rug Warrior"                                                                              */
/* File   : "DRIVE.C"                                                                                                      */
/* Author : Lars Ferner / Robin Wissbrock                                                                                  */
/* Date   : 96-06-12 / 95-12-07                                                                                            */
/* Notes  : Utility functions for monitoring the shaft encoders                                                            */
/*          ACHTUNG: - alle Funktionen dieses Moduls koennen falsche Ergebnisse liefern, wenn die Zaehlerstaende waehrend  */
/*                     des Auslesens durch eine Interrupt-Routine veraendert werden -> alles in Assembler implementieren   */
/*                     und Interrupts voruebergehend ausschalten                                                           */
/*                   - in dieser Datei sind neuerdings auch die Routinen zum Ansteuern der Motoren enthalten, weil diese   */
/*                     auf Assembler-Routinen zugreifen, die fuer die Beseitigung eines Fehlers beim Herabsetzen des       */
/*                     Zaehlerwertes fuer das PWM-Signal ntig und beim Laden der Library noch nicht verfuegbar sind       */


/* Global constants                                                                                                        */

float WHEEL_TRACK = 0.14;
float WHEEL_GIRTH = 0.2;
float MARKINGS = 16.0;
float DISTANCE_PER_CLICK = WHEEL_GIRTH / MARKINGS;


/* Global variables                                                                                                        */

int left_distance, right_distance;

/*                  Left        Right                         Setup two PWM channels for motor control                     */
int TOCx[2]     = { 0x1018    , 0x101A     };              /* Index for timer register                                     */
int pwm_mask[2] = { 0b01000000, 0b00100000 };              /* Port A PWM bits                                              */
int dir_mask[2] = { 0b00010000, 0b00100000 };              /* Port D direction bits                                        */


/* Functions (definitions)                                                                                                 */

int init_motors( void )
{
	bit_set( 0x1009, 0b110000 );                            /* Set PD4,5 as outputs for motor direction                     */
	poke( 0x100C, 0b01100000 );                             /* OC1M Output compare 1 affects PA5,6                          */
	bit_set( 0x1020, 0b10100000 );                          /* TCTL1 OC3 turns off PA5, OC2 PA6                             */
	bit_clear( 0x1020, 0b01010000 );                        /* Use set and clear to avoid other bits                        */
	pokeword( 0x1018, 0 );                                  /* When TCNT = 0, OC1 fires                                     */
}

void motor( int index, int vel )
/* Vel is in the range [-100, +100], index = 0 => Left, = 1 => Right                                                       */
{
	/* Local variables                                                                                                      */

	int avel;                                               /* Absolute value of velocity                                   */


	if ( vel > 0 )
	{
		bit_set( 0x1008, dir_mask[index] );                  /* Forward rotation                                             */
		avel = vel;
	}
	else
	{
		bit_clear( 0x1008, dir_mask[index] );                /* Backward rotation                                            */
		avel = -vel;
	}

	if ( avel < 5 )                                         /* If we are going real slow                                    */
		bit_clear( 0x100D, pwm_mask[index] );                /* then just have OC1 turn off the motor                        */
	else
		bit_set( 0x100D,pwm_mask[index] );                   /* Otherwise let OC1 control the motors                         */

	if ( avel > 99 )                                        /* If we are going real fast                                    */
		avel = 99;                                           /* don't let the following multiply overflow                    */

	avel = (int) ( 655.56 * (float) avel );
	pokeword( TOCx[index], avel );                          /* Compute TOCx value                                           */

	if ( index == LEFT )                                    /* wenn neuer Zaehlerwert bereits berschritten, Spannung fuer  */
		ClearSignal_0( avel );                               /* entspr. Motor manuell abschalten                             */
	else
		ClearSignal_1( avel );
}

void init_velocity( void )
/* Enable PA7 pulse counting and PA0 interrupt                                                                             */
{
	poke( 0x1026, 0b01010000 );                             /* PACTL PA7 input, trigger on rising edge                      */
	poke( 0x1027,0 );                                       /* PACNT init to 0                                              */
	left_clicks = 0;                                        /* Global referenced by interrupt routine                       */
	left_distance = 0;                                      /* mit dem linken Rad zurueckgelegte Strecke (in 'clicks')      */
	right_distance = 0;                                     /* mit dem rechten Rad zurueckgelegte Strecke (in 'clicks')     */
	bit_clear( 0x1021, 0b10 );                              /* TCTL2 Assign values to the two lowest order bits             */
	bit_set( 0x1021, 0b01 );                                /* of tctl2 without changing any other bits                     */
	bit_set( 0x1022, 0b01 );                                /* TMSK1 enable IC3 interrupts                                  */
}

int get_left_clicks( void )
/* Get the number of clicks and reset to 0                                                                                 */
/* If called at regular intervals this funciton returns the left velocity in units of clicks per interval                  */
{
	/* Local variables                                                                                                      */

	int clks = left_clicks;


	left_clicks = 0;
	left_distance += clks;

	return ( clks );
}

int get_right_clicks( void )
/* Same as above for right Shaft encoder                                                                                   */
/* PACNT is only an 8-bit register -- be sure to call it often enough so that it dosen't overflow                          */
{
	/* Local variables                                                                                                      */

	int clks = peek( 0x1027 );


	poke( 0x1027, 0 );
	right_distance += clks;

	return ( clks );
}

float odometer( int wheel )
/* "Tageskilometerzaehler": bestimmt die vom linken resp. rechten Rad seit dem letzten Reset zurueckgel. Entfernung in cm  */
/* ACHTUNG: Die Summenbildung ist problematisch, weil das zurueckgelieferte Ergebnis trotz eines 'odometer_reset' immer    */
/*          groesser wird, wenn nicht zwischendurch auch die Funktion 'get_left/right_clicks' aufgerufen wird!!!           */
{
	if ( wheel == LEFT )
		return ( DISTANCE_PER_CLICK * (float) ( left_distance + left_clicks ) );
	else
		return ( DISTANCE_PER_CLICK * (float) ( right_distance + peek( 0x1027 ) ) );
}

void reset_odometer( int wheel )
/* setzt den linken bzw. rechten "Tageskilometerzaehler" auf Null                                                          */
{
/*	get_left_clicks();
	get_right_clicks();
*/
	if ( wheel == LEFT )
		left_distance = 0;
	else
		right_distance = 0;
}

